/****************************************************************************
** Meta object code from reading C++ file 'calendarmanage.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dataManage/calendarmanage.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'calendarmanage.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CalendarManager_t {
    QByteArrayData data[11];
    char stringdata0[211];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CalendarManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CalendarManager_t qt_meta_stringdata_CalendarManager = {
    {
QT_MOC_LITERAL(0, 0, 15), // "CalendarManager"
QT_MOC_LITERAL(1, 16, 23), // "signalTimeFormatChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 5), // "value"
QT_MOC_LITERAL(4, 47, 23), // "signalDateFormatChanged"
QT_MOC_LITERAL(5, 71, 31), // "sigNotifySidebarFirstDayChanged"
QT_MOC_LITERAL(6, 103, 17), // "weekBeginsChanged"
QT_MOC_LITERAL(7, 121, 19), // "slotGetLunarSuccess"
QT_MOC_LITERAL(8, 141, 25), // "slotGeneralSettingsUpdate"
QT_MOC_LITERAL(9, 167, 21), // "slotDateFormatChanged"
QT_MOC_LITERAL(10, 189, 21) // "slotTimeFormatChanged"

    },
    "CalendarManager\0signalTimeFormatChanged\0"
    "\0value\0signalDateFormatChanged\0"
    "sigNotifySidebarFirstDayChanged\0"
    "weekBeginsChanged\0slotGetLunarSuccess\0"
    "slotGeneralSettingsUpdate\0"
    "slotDateFormatChanged\0slotTimeFormatChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CalendarManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       4,    1,   57,    2, 0x06 /* Public */,
       5,    1,   60,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   63,    2, 0x0a /* Public */,
       7,    0,   66,    2, 0x0a /* Public */,
       8,    0,   67,    2, 0x0a /* Public */,
       9,    1,   68,    2, 0x0a /* Public */,
      10,    1,   71,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,

       0        // eod
};

void CalendarManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CalendarManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalTimeFormatChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->signalDateFormatChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->sigNotifySidebarFirstDayChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->weekBeginsChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->slotGetLunarSuccess(); break;
        case 5: _t->slotGeneralSettingsUpdate(); break;
        case 6: _t->slotDateFormatChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->slotTimeFormatChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CalendarManager::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CalendarManager::signalTimeFormatChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CalendarManager::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CalendarManager::signalDateFormatChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CalendarManager::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CalendarManager::sigNotifySidebarFirstDayChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CalendarManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CalendarManager.data,
    qt_meta_data_CalendarManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CalendarManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CalendarManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CalendarManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CalendarManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void CalendarManager::signalTimeFormatChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CalendarManager::signalDateFormatChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void CalendarManager::sigNotifySidebarFirstDayChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
