/****************************************************************************
** Meta object code from reading C++ file 'dbuscalendar_adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbus/dbuscalendar_adaptor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbuscalendar_adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CalendarAdaptor_t {
    QByteArrayData data[24];
    char stringdata0[1746];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CalendarAdaptor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CalendarAdaptor_t qt_meta_stringdata_CalendarAdaptor = {
    {
QT_MOC_LITERAL(0, 0, 15), // "CalendarAdaptor"
QT_MOC_LITERAL(1, 16, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 32, 19), // "com.deepin.Calendar"
QT_MOC_LITERAL(3, 52, 19), // "D-Bus Introspection"
QT_MOC_LITERAL(4, 72, 1478), // "  <interface name=\"com.deepi..."
QT_MOC_LITERAL(5, 1399, 12), // "ActiveWindow"
QT_MOC_LITERAL(6, 1412, 0), // ""
QT_MOC_LITERAL(7, 1413, 11), // "RaiseWindow"
QT_MOC_LITERAL(8, 1425, 12), // "OpenSchedule"
QT_MOC_LITERAL(9, 1438, 3), // "job"
QT_MOC_LITERAL(10, 1442, 14), // "QuerySchedules"
QT_MOC_LITERAL(11, 1457, 5), // "query"
QT_MOC_LITERAL(12, 1463, 14), // "CreateSchedule"
QT_MOC_LITERAL(13, 1478, 12), // "scheduleData"
QT_MOC_LITERAL(14, 1491, 14), // "ModifySchedule"
QT_MOC_LITERAL(15, 1506, 10), // "scheduleId"
QT_MOC_LITERAL(16, 1517, 9), // "operation"
QT_MOC_LITERAL(17, 1527, 4), // "data"
QT_MOC_LITERAL(18, 1532, 15), // "GetCalendarView"
QT_MOC_LITERAL(19, 1548, 8), // "viewType"
QT_MOC_LITERAL(20, 1557, 4), // "date"
QT_MOC_LITERAL(21, 1562, 12), // "GetLunarInfo"
QT_MOC_LITERAL(22, 1575, 12), // "GetReminders"
QT_MOC_LITERAL(23, 1588, 5) // "hours"

    },
    "CalendarAdaptor\0D-Bus Interface\0"
    "com.deepin.Calendar\0D-Bus Introspection\0"
    "  <interface name=\"com.deepin.Calendar\">\n    <method name=\"ActiveW"
    "indow\">\n    </method>\n    <method name=\"RaiseWindow\">\n    </meth"
    "od>\n    <method name=\"OpenSchedule\">\n      <arg direction=\"in\" t"
    "ype=\"s\" name=\"job\"/>\n    </method>\n    <method name=\"QuerySched"
    "ules\">\n      <arg direction=\"in\" type=\"s\" name=\"query\"/>\n    "
    "  <arg direction=\"out\" type=\"s\" name=\"result\"/>\n    </method>\n"
    "    <method name=\"CreateSchedule\">\n      <arg direction=\"in\" type"
    "=\"s\" name=\"scheduleData\"/>\n      <arg direction=\"out\" type=\"s\""
    " name=\"scheduleId\"/>\n    </method>\n    <method name=\"ModifySchedu"
    "le\">\n      <arg direction=\"in\" type=\"s\" name=\"scheduleId\"/>\n "
    "     <arg direction=\"in\" type=\"s\" name=\"operation\"/>\n      <arg"
    " direction=\"in\" type=\"s\" name=\"data\"/>\n      <arg direction=\"o"
    "ut\" type=\"b\" name=\"success\"/>\n    </method>\n    <method name=\""
    "GetCalendarView\">\n      <arg direction=\"in\" type=\"s\" name=\"view"
    "Type\"/>\n      <arg direction=\"in\" type=\"s\" name=\"date\"/>\n    "
    "  <arg direction=\"out\" type=\"s\" name=\"viewData\"/>\n    </method>"
    "\n    <method name=\"GetLunarInfo\">\n      <arg direction=\"in\" type"
    "=\"s\" name=\"date\"/>\n      <arg direction=\"out\" type=\"s\" name=\""
    "lunarData\"/>\n    </method>\n    <method name=\"GetReminders\">\n    "
    "  <arg direction=\"in\" type=\"i\" name=\"hours\"/>\n      <arg direct"
    "ion=\"out\" type=\"s\" name=\"reminders\"/>\n    </method>\n  </interf"
    "ace>\n\0"
    "ActiveWindow\0\0RaiseWindow\0OpenSchedule\0"
    "job\0QuerySchedules\0query\0CreateSchedule\0"
    "scheduleData\0ModifySchedule\0scheduleId\0"
    "operation\0data\0GetCalendarView\0viewType\0"
    "date\0GetLunarInfo\0GetReminders\0hours"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CalendarAdaptor[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       9,   18, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // slots: name, argc, parameters, tag, flags
       5,    0,   63,    6, 0x0a /* Public */,
       7,    0,   64,    6, 0x0a /* Public */,
       8,    1,   65,    6, 0x0a /* Public */,
      10,    1,   68,    6, 0x0a /* Public */,
      12,    1,   71,    6, 0x0a /* Public */,
      14,    3,   74,    6, 0x0a /* Public */,
      18,    2,   81,    6, 0x0a /* Public */,
      21,    1,   86,    6, 0x0a /* Public */,
      22,    1,   89,    6, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::QString, QMetaType::QString,   11,
    QMetaType::QString, QMetaType::QString,   13,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString, QMetaType::QString,   15,   16,   17,
    QMetaType::QString, QMetaType::QString, QMetaType::QString,   19,   20,
    QMetaType::QString, QMetaType::QString,   20,
    QMetaType::QString, QMetaType::Int,   23,

       0        // eod
};

void CalendarAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CalendarAdaptor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->ActiveWindow(); break;
        case 1: _t->RaiseWindow(); break;
        case 2: _t->OpenSchedule((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: { QString _r = _t->QuerySchedules((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->CreateSchedule((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->ModifySchedule((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->GetCalendarView((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->GetLunarInfo((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { QString _r = _t->GetReminders((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CalendarAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_meta_stringdata_CalendarAdaptor.data,
    qt_meta_data_CalendarAdaptor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CalendarAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CalendarAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CalendarAdaptor.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int CalendarAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
