/****************************************************************************
** Meta object code from reading C++ file 'dbusaccountmanagerrequest.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../src/dbus/dbusaccountmanagerrequest.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusaccountmanagerrequest.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DbusAccountManagerRequest_t {
    QByteArrayData data[15];
    char stringdata0[256];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DbusAccountManagerRequest_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DbusAccountManagerRequest_t qt_meta_stringdata_DbusAccountManagerRequest = {
    {
QT_MOC_LITERAL(0, 0, 25), // "DbusAccountManagerRequest"
QT_MOC_LITERAL(1, 26, 26), // "signalGetAccountListFinish"
QT_MOC_LITERAL(2, 53, 0), // ""
QT_MOC_LITERAL(3, 54, 14), // "DAccount::List"
QT_MOC_LITERAL(4, 69, 11), // "accountList"
QT_MOC_LITERAL(5, 81, 30), // "signalGetGeneralSettingsFinish"
QT_MOC_LITERAL(6, 112, 29), // "DCalendarGeneralSettings::Ptr"
QT_MOC_LITERAL(7, 142, 3), // "ptr"
QT_MOC_LITERAL(8, 146, 27), // "signalGetIsSupportUidFinish"
QT_MOC_LITERAL(9, 174, 9), // "supported"
QT_MOC_LITERAL(10, 184, 16), // "slotCallFinished"
QT_MOC_LITERAL(11, 201, 24), // "CDBusPendingCallWatcher*"
QT_MOC_LITERAL(12, 226, 12), // "slotDbusCall"
QT_MOC_LITERAL(13, 239, 12), // "QDBusMessage"
QT_MOC_LITERAL(14, 252, 3) // "msg"

    },
    "DbusAccountManagerRequest\0"
    "signalGetAccountListFinish\0\0DAccount::List\0"
    "accountList\0signalGetGeneralSettingsFinish\0"
    "DCalendarGeneralSettings::Ptr\0ptr\0"
    "signalGetIsSupportUidFinish\0supported\0"
    "slotCallFinished\0CDBusPendingCallWatcher*\0"
    "slotDbusCall\0QDBusMessage\0msg"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DbusAccountManagerRequest[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,
       5,    1,   42,    2, 0x06 /* Public */,
       8,    1,   45,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   48,    2, 0x0a /* Public */,
      12,    1,   51,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::Bool,    9,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, 0x80000000 | 13,   14,

       0        // eod
};

void DbusAccountManagerRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DbusAccountManagerRequest *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signalGetAccountListFinish((*reinterpret_cast< DAccount::List(*)>(_a[1]))); break;
        case 1: _t->signalGetGeneralSettingsFinish((*reinterpret_cast< DCalendarGeneralSettings::Ptr(*)>(_a[1]))); break;
        case 2: _t->signalGetIsSupportUidFinish((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->slotCallFinished((*reinterpret_cast< CDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 4: _t->slotDbusCall((*reinterpret_cast< const QDBusMessage(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< CDBusPendingCallWatcher* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DbusAccountManagerRequest::*)(DAccount::List );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountManagerRequest::signalGetAccountListFinish)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DbusAccountManagerRequest::*)(DCalendarGeneralSettings::Ptr );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountManagerRequest::signalGetGeneralSettingsFinish)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DbusAccountManagerRequest::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DbusAccountManagerRequest::signalGetIsSupportUidFinish)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DbusAccountManagerRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<DbusRequestBase::staticMetaObject>(),
    qt_meta_stringdata_DbusAccountManagerRequest.data,
    qt_meta_data_DbusAccountManagerRequest,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DbusAccountManagerRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DbusAccountManagerRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DbusAccountManagerRequest.stringdata0))
        return static_cast<void*>(this);
    return DbusRequestBase::qt_metacast(_clname);
}

int DbusAccountManagerRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DbusRequestBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void DbusAccountManagerRequest::signalGetAccountListFinish(DAccount::List _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DbusAccountManagerRequest::signalGetGeneralSettingsFinish(DCalendarGeneralSettings::Ptr _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DbusAccountManagerRequest::signalGetIsSupportUidFinish(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
