// SPDX-FileCopyrightText: 2019 - 2026 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef QUERYJSONDATA_H
#define QUERYJSONDATA_H

#include "jsondata.h"

class QueryJsonData : public JsonData
{
public:
    QueryJsonData();
};

#endif // QUERYJSONDATA_H
