/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -c ControlCenterProxy -p controlCenterProxy /build/deepin-calendar/src/dde-calendar/src/calendar-client/src/dbus/org.deepin.dde.ControlCenter.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "controlCenterProxy.h"

/*
 * Implementation of interface class ControlCenterProxy
 */

ControlCenterProxy::ControlCenterProxy(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

ControlCenterProxy::~ControlCenterProxy()
{
}


#include "controlCenterProxy.moc"
